* This program calculates the hourly loads, capacity outage probability table, hourly LOLP's, and annual LOLE for the 1979 IEEE RTS.
* This program was written by Eugene G. Preston  May 11, 2016  and is posted on  http://egpreston.com/RTS1979.txt .
* The program produces exactly the same LOLE and LOLH as shown in the 1986 RTS paper on page 2.
* Source of all the data below is from  https://www.ee.washington.edu/research/pstca/rts/rts79/ieeerts79.txt .
      REAL WPD(52)/.862,.900,.878,.834,.880,.841,.832,.806,.740,.737,    ! Weekly Peak Demand for a 2850 MW annual peak
     &   .715,.727,.704,.750,.721,.800,.754,.837,.870,.880,.856,.811,
     &   .900,.887,.896,.861,.755,.816,.801,.880,.722,.776,.800,.729,
     &   .726,.705,.780,.695,.724,.724,.743,.744,.800,.881,.885,.909,
     &   .940,.890,.942,.970,1.00,.952/
      REAL DPD(7)/.93,1.00,.98,.96,.94,.77,.75/,HD(8736),FOR(9)          ! Daily Peak Demand each week, hourly demands, forced outage rates
      REAL SHD(24,6)/.67,.63,.60,.59,.59,.60,.74,.86,.95,.96,.96,.95,    ! Winter Weekday AM       SEA 1
     &               .95,.95,.93,.94,.99,1.0,1.0,.96,.91,.83,.73,.63,    ! Winter Weekday PM
     &               .78,.72,.68,.66,.64,.65,.66,.70,.80,.88,.90,.91,    ! Winter Weekend AM       SEA 2
     &               .90,.88,.87,.87,.91,1.0,.99,.97,.94,.92,.87,.81,    ! Winter Weekend PM
     &               .64,.60,.58,.56,.56,.58,.64,.76,.87,.95,.99,1.0,    ! Summer Weekday AM       SEA 3
     &               .99,1.0,1.0,.97,.96,.96,.93,.92,.92,.93,.87,.72,    ! Summer Weekday PM
     &               .74,.70,.66,.65,.64,.62,.62,.66,.81,.86,.91,.93,    ! Summer Weekend AM       SEA 4
     &               .93,.92,.91,.91,.92,.94,.95,.95,1.0,.93,.88,.80,    ! Summer Weekend PM
     &               .63,.62,.60,.58,.59,.65,.72,.85,.95,.99,1.0,.99,    ! Spring/Fall Weekday AM  SEA 5
     &               .93,.92,.90,.88,.90,.92,.96,.98,.96,.90,.80,.70,    ! Spring/Fall Weekday PM
     &               .75,.73,.69,.66,.65,.65,.68,.74,.83,.89,.92,.94,    ! Spring/Fall Weekend AM  SEA 6
     &               .91,.90,.90,.86,.85,.88,.92,1.0,.97,.95,.90,.85/    ! Spring/Fall Weekend PM
      INTEGER SEA(52)/1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,3,3,3,3,3,3,3,3, ! Season of each week
     &            3,3,3,3,3,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,1,1,1,1,1/
      INTEGER CAPA(9)/  12,  20,  50,  76, 100, 155, 197, 350, 400/,     ! generator MW capacities
     &        NUMB(9)/   5,   4,   6,   4,   3,   4,   3,   1,   2/,     ! number of generators per type
     &        MTTF(9)/2940, 450,1980,1960,1200, 960, 950,1150,1100/,     ! mean time to failure in hours
     &        MTTR(9)/  60,  50,  20,  40,  50,  40,  50, 100, 150/      ! mean time to repair in hours
      REAL F(-1:3000)/1.,3001*0./,LOLP                                   ! F is the capacity outage probability table COPT  LOLP=loss of load probability
      REAL*8 LOLE/0.D0/,LOLH/0.D0/                                       ! loss of load expectation and loss of load hours are in double precision
      OPEN(5,FILE='RTSPU.TXT')
      PRINT *,'SEE RTS PU LOADS IN FILE RTSPU.TXT'
* calculate generator forced outage rates
      DO 1 I=1,9                                                         ! F(0)=prob at MAXGEN so that LOLP=F(MAXGEN-HD) where HD is the hourly demand
   1  FOR(I)=FLOAT(MTTR(I))/(MTTR(I)+MTTF(I))                            ! calculate the different generator type FOR's forced outage rates
      PRINT *,'FORs:',FOR
* calculate hourly demands HD
      I=0                                                                ! hour counter for creating the HD hourly demands
      DO 2 N=1,30
      DO 2 J=1,52                                                        ! week number
      DO 2 K=1,7                                                         ! day within the week number
      ISEA=SEA(J)                                                        ! assume weekday
      IF(K.GE.6) ISEA=SEA(J)+1                                           ! change to weekend
      DO 2 L=1,24                                                        ! hour within the day
      I=I+1                                                              ! the hour counter
      PUD=WPD(J)*DPD(K)*SHD(L,ISEA)                                      ! hourly per unit demand from seasonal, weekend-weekday, AM-PM factors
      IF(N.EQ.1) THEN
        HD(I)=2850.*PUD                                                  ! hourly load from seasonal, weekend-weekday, AM-PM factors
        WRITE(5,'(F10.7,F10.3)') PUD,HD(I)                               ! only first year writes out the MW values
      ELSE
        WRITE(5,'(F10.7)') PUD                                           ! 30 years of RTS PU load
      ENDIF
   2  CONTINUE
* calculate the COPT, the capacity outage probability table F
      MAXGEN=0                                                           ! MAXGEN is the sum of CAPA = total installed generation MW
      DO 3 K=1,9                                                         ! 9 kinds of generators
      DO 3 I=1,NUMB(K)                                                   ! number of generators of each kind or type
      MAXGEN=MAXGEN+CAPA(K)                                              ! MAXGEN is the total installed generation MW
      DO 3 IX=3000,0,-1                                                  ! sweep from right to left across F to convolve the next generator
      J=IX-CAPA(K)                                                       ! J will point to the shifted to the right generator outage state
      IF(J.LT.0) J=-1                                                    ! all F(IX<0) = 1, we store the 1 in F(-1) which stays constant
      F(IX)=(1.-FOR(K))*F(IX)+FOR(K)*F(J)                                ! down state is shifted to the right and added to the up state which was not moved
      IF(F(IX).LT.1.E-37) F(IX)=0.                                       ! this prevents an underflow error message
   3  CONTINUE
* sweep through all the hours calculating the LOLP each hour and the LOLE and LOLH
      IHRS=0                                                             ! count the hours per day and reset to define a new day, capture the max LOLP
      PLOLP=0.                                                           ! PLOLP will be the peak LOLP of each day
      DO 4 IH=1,8736
      IHD=HD(IH)                                                         ! this is the integer part of HD to the left of the decimal
      P=HD(IH)-IHD                                                       ! P is the fraction of MW < 1 MW greater than the integer HD MW
      LOLP=0.
      IF(MAXGEN-IHD.LE.3000) THEN                                        ! assume F(x>1000)=0
        IF(P.GT.0.) IHD=IHD+1                                            ! pick up the LOLP for the next higher MW load level, its a stairstep
        LOLP=F(MAXGEN-IHD)                                               ! the LOLP is a lookup
      ENDIF
      LOLH=LOLH+LOLP                                                     ! calculate the annual loss of load hours
      IHRS=IHRS+1                                                        ! this is the hour of the day
      IF(LOLP.GT.PLOLP) PLOLP=LOLP                                       ! capture the max LOLP each day
      IF(IHRS.EQ.24) THEN                                                ! we are at the end of a day
        LOLE=LOLE+PLOLP                                                  ! the LOLE is the sum of daily maximum LOLPs
        PLOLP=0.                                                         ! reset PLOLP and IHRS values for a new day
        IHRS=0
      ENDIF
   4  CONTINUE
      PRINT *,'LOLE   =',LOLE                                            ! LOLE   =   1.368862 d/y which is exactly the 1986 paper result
      PRINT *,'LOLH   =',LOLH                                            ! LOLH   =   9.394185 h/y which is exactly the 1986 paper result
      PRINT *,'MAXGEN =',MAXGEN
      PRINT *,'LOLP PK=',F(MAXGEN-2850)
      STOP
      END
